Attribute VB_Name = "mdlPrayer"
'****************************************************
'Version        :1.0.0
'Last update    :13/07/2007
'****************************************************
'Author         :Chenafi Moulay Abd el-kader djilali
'Adress         :Aflou 03400 - Algeria
'E-mail         :m-chenafi@caramail.com
'Forum          :www.montada.com
'               :www.vb4arab.com
'****************************************************
'Description    :
'****************************************************
Option Explicit
Private Declare Function GetLocaleInfo Lib "kernel32" Alias "GetLocaleInfoA" (ByVal Locale As Long, ByVal LCType As Long, ByVal lpLCData As String, ByVal cchData As Long) As Long
Private Declare Function GetUserDefaultLCID Lib "kernel32" () As Long
Private Const LOCALE_SDECIMAL = &HE
Public PrayerIndex As Integer
Public Prayers(5) As Date
Private Latitude As String
Private Longitude As String
Private Gmt As String
Private Summer As String
Private Sobh As String
Private Asr As String
Private Isha As String
Private Function GetLocalSetting(ByVal Parametre As Long) As String
    Dim lngResultat As Long
    Dim Buffer As String
    Dim Pos As Integer
    Dim Locale As Long
    Locale = GetUserDefaultLCID()
    lngResultat = GetLocaleInfo(Locale, Parametre, Buffer, 0)
    Buffer = String(lngResultat, 0)
    GetLocaleInfo Locale, Parametre, Buffer, lngResultat
    Pos = InStr(Buffer, Chr(0))
    If Pos > 0 Then GetLocalSetting = Left(Buffer, Pos - 1)
End Function
Private Sub GetParametres()
    Dim Sdecimal As String
    Sdecimal = GetLocalSetting(LOCALE_SDECIMAL)
    Latitude = ReadInit("Options", "Latitude", App.Path & "\Setting.inf", "")
    Longitude = ReadInit("Options", "Longitude", App.Path & "\Setting.inf", "")
    Gmt = ReadInit("Options", "Gmt", App.Path & "\Setting.inf", "")
    If Date >= CDate("21/06/" & Year(Date)) And Date <= CDate("21/09/" & Year(Date)) Then Summer = ReadInit("Options", "Summer", App.Path & "\Setting.inf", "")
    Sobh = ReadInit("Options", "Sobh", App.Path & "\Setting.inf", "")
    Asr = ReadInit("Options", "Asr", App.Path & "\Setting.inf", "")
    Isha = ReadInit("Options", "Isha", App.Path & "\Setting.inf", "")
    Latitude = Replace(Latitude, Sdecimal, ".")
    Longitude = Replace(Longitude, Sdecimal, ".")
    Sobh = Replace(Sobh, Sdecimal, ".")
    Isha = Replace(Isha, Sdecimal, ".")
End Sub
Public Function Sinus(ByVal Degree As Double) As Double
    Dim Pi As Double: Dim Rad As Double: Dim Deg As Double
    Pi = 4 * Atn(1): Rad = Pi / 180: Deg = 180 / Pi
    Sinus = Sin(Rad * Degree)
End Function
Public Function Cosinus(ByVal Degree As Double) As Double
    Dim Pi As Double: Dim Rad As Double: Dim Deg As Double
    Pi = 4 * Atn(1): Rad = Pi / 180: Deg = 180 / Pi
    Cosinus = Cos(Rad * Degree)
End Function
Private Function GetJulianDay(ByVal iDate As Date) As Double
    Dim a As Double
    Dim b As Double
    Dim y As Double
    Dim m As Double
    Dim d As Double
    y = Year(iDate)
    m = Month(iDate)
    d = Day(iDate)
    If m < 3 Then
        y = y - 1
        m = m + 12
    End If
    If CDate(iDate) > #10/15/1582# Then
        a = Fix(y / 100)
        b = 2 - a + Fix(a / 4)
    End If
    GetJulianDay = Fix(365.25 * (y + 4716)) + Fix(30.6001 * (m + 1)) + d + b - 1524.5
End Function
Private Function GetEquationTime(ByVal iDate As Date) As Double
    Const M0 As Double = 357.5291
    Const M1 As Double = 0.98560028
    Const J2000 As Double = 2451544.5
    Dim Jd As Double
    Dim m As Double
    Dim C As Double
    Dim Lambda As Double
    Dim R As Double
    Jd = GetJulianDay(iDate)
    m = M0 + M1 * (Jd - J2000)
    C = 1.9148 * Sinus(m) + 0.02 * Sinus(2 * m) + 0.0003 * Sinus(3 * m)
    Lambda = 280.47 + M1 * (Jd - J2000) + C
    R = -2.468 * Sinus(2 * Lambda) + 0.053 * Sinus(4 * Lambda) + 0.0014 * Sinus(6 * Lambda)
    GetEquationTime = (C + R) * 4
End Function
Private Function GetSunDeclination(ByVal iDate As Date) As Double
    Const J2000 As Double = 2451544.5
    Dim Jd As Double
    Jd = GetJulianDay(iDate)
    Dim Epsilon As Double
    Epsilon = 23.44 - 0.0000004 * (Jd - J2000)
    Dim Lamda As Double
    Dim LD As Double
    LD = 280.466 + 0.9856474 * (Jd - J2000)
    Dim GD As Double
    GD = 357.528 + 0.9856003 * (Jd - J2000)
    Lamda = LD + 1.915 * Sinus(GD) + 0.02 * Sinus(2 * GD)
    Dim X As Double
    X = Sinus(Epsilon) * Sinus(Lamda)
    GetSunDeclination = (180 / (4 * Atn(1))) * Atn(X / Sqr(-X * X + 1))
End Function
Private Function GetZenith(ByVal iDate As Date) As Double
    Dim Sd As Double
    Dim X As Double
    Dim a As Double
    Sd = GetSunDeclination(iDate)
    X = Sinus(Val(Latitude)) * Sinus(Sd) + Cosinus(Val(Latitude)) * Cosinus(Sd)
    a = Atn(X / Sqr(-X * X + 1))
    X = Val(Asr) + (1 / Tan(a))
    GetZenith = 90 - (180 / (4 * Atn(1))) * (Atn(X) + 2 * Atn(1))
End Function
Private Function GetDohrTime(ByVal iDate As Date) As Boolean
    On Error GoTo ErrHndlr
    Dim Mt As Double
    Dim Dl As Double
    Dim Et As Double
    Mt = Val(Gmt) * 15
    Dl = (Mt - Val(Longitude)) / 15
    Et = GetEquationTime(iDate)
    Dim h As Double
    Dim m As Double
    Dim s As Double
    h = 12 + Dl + Et / 60
    m = (h - Fix(h)) * 60
    s = (m - Fix(m)) * 60
    Prayers(2) = CDate(Fix(h + Val(Summer)) & ":" & Fix(m) & ":" & Fix(s))
    GetDohrTime = True
    Exit Function
ErrHndlr:
    Prayers(2) = 0
    GetDohrTime = False
End Function
Private Function GetDeltaTime(ByVal Zenith As Double, ByVal iDate As Date) As Date
    On Error GoTo ErrHndlr
    Dim Sd As Double
    Sd = GetSunDeclination(iDate)
    Dim X As Double
    X = (Cosinus(Zenith) - Sinus(Val(Latitude)) * Sinus(Sd)) / (Cosinus(Val(Latitude)) * Cosinus(Sd))
    Dim y As Double
    y = 180 / (4 * Atn(1)) * (Atn(-X / Sqr(-X * X + 1)) + 2 * Atn(1))
    Dim h As Double
    Dim m As Double
    Dim s As Double
    h = y / 15
    m = (h - Fix(h)) * 60
    s = (m - Fix(m)) * 60
    Dim Delta As Date
    Delta = CDate(Fix(h) & ":" & Fix(m) & ":" & Fix(s))
    GetDeltaTime = Delta
    Exit Function
ErrHndlr:
    GetDeltaTime = 0
End Function
Public Sub GetPrayerTimes(ByVal iDate As Date)
    GetParametres
    If GetDohrTime(iDate) = True Then
        Prayers(0) = Prayers(2) - GetDeltaTime(Val(Sobh), iDate)
        Prayers(1) = Prayers(2) - GetDeltaTime(90.83333, iDate)
        Prayers(3) = Prayers(2) + GetDeltaTime(GetZenith(iDate), iDate)
        Prayers(4) = Prayers(2) + GetDeltaTime(90.83333, iDate)
        If Val(ReadInit("Options", "Organization", App.Path & "\Setting.inf", "")) = 4 Then
            If Month(ConvertDate(iDate, 0)) = 9 Then
                Prayers(5) = DateAdd("n", 120, Prayers(4))
            Else
                Prayers(5) = DateAdd("n", 90, Prayers(4))
            End If
        Else
            Prayers(5) = Prayers(2) + GetDeltaTime(Val(Isha), iDate)
        End If
    Else
        Prayers(0) = 0
        Prayers(1) = 0
        Prayers(3) = 0
        Prayers(4) = 0
        Prayers(5) = 0
        MsgBox Err.Number & Err.Description
    End If
End Sub
Public Function GetQuiblaDirection() As Double
    Const MakaLat As Double = 21.4266667
    Const MakaLog As Double = 39.8261111
    Dim Lambda As Double
    Lambda = MakaLog - Val(Longitude)
    Dim Numerator As Double
    Dim Denominator As Double
    Numerator = Cosinus(MakaLat) * Sinus(Lambda)
    Denominator = Sinus(MakaLat) * Cosinus(Val(Latitude)) - Cosinus(MakaLat) * Sinus(Val(Latitude)) * Cosinus(Lambda)
    Dim X As Double
    X = Numerator / Denominator
    Dim Q As Double
    Q = (180 / (4 * Atn(1))) * Atn(X)
    If Numerator > 0 And Denominator < 0 Then Q = 180 + Q
    If Numerator < 0 And Denominator < 0 Then Q = 180 + Q
    If Numerator < 0 And Denominator > 0 Then Q = 360 + Q
    GetQuiblaDirection = Q
End Function
Public Function ConvertDate(ByVal iDate As Date, ByVal Clndr As Integer) As String
    Dim OldClndr As Integer
    Dim Buffer As String
    OldClndr = Calendar
    If Clndr = 0 Then Calendar = vbCalHijri Else Calendar = vbCalGreg
    Buffer = iDate
    Calendar = OldClndr
    ConvertDate = Buffer
End Function

