Attribute VB_Name = "MPathRoutines"
'*******************************************************************************
' MODULE:       MPathRoutines
' FILENAME:     C:\My Code\vb\Misc\PathRoutines\MPathRoutines.bas
' AUTHOR:       Phil Fresle
' CREATED:      13-Jan-2000
' COPYRIGHT:    Copyright 2000 Frez Systems Limited. All Rights Reserved.
'
' DESCRIPTION:
' Routines for path manipulation using the shlwapi.dll type library that comes
' with Internet Explorer.
'
' GetRelativePath demonstrates when given a directory path and a file path how
' to return the relative path from the directory to the file.
'
' GetCompactPath demonstartes how to compact a long path name down into a small
' space in order to display it effectively in, say, a small text box.
'
' This is 'free' software with the following restrictions:
'
' You may not redistribute this code as a 'sample' or 'demo'. However, you are free
' to use the source code in your own code, but you may not claim that you created
' the sample code. It is expressly forbidden to sell or profit from this source code
' other than by the knowledge gained or the enhanced value added by your own code.
'
' Use of this software is also done so at your own risk. The code is supplied as
' is without warranty or guarantee of any kind.
'
' Should you wish to commission some derivative work based on the code provided
' here, or any consultancy work, please do not hesitate to contact us.
'
' Web Site:  http://www.frez.co.uk
' E-mail:    sales@frez.co.uk
'
' MODIFICATION HISTORY:
' 1.0       13-Jan-2000
'           Phil Fresle
'           Initial Version
'*******************************************************************************
Option Explicit

Private Declare Function PathRelativePathToW Lib "shlwapi.dll" _
    (ByVal pszPath As Long, _
     ByVal pszFrom As Long, _
     ByVal dwAttrFrom As Long, _
     ByVal pszTo As Long, _
     ByVal dwAttrTo As Long) As Boolean
     
Private Declare Function PathCompactPathW Lib "shlwapi.dll" _
    (ByVal hdc As Long, _
     ByVal lpszPath As Long, _
     ByVal dx As Long) As Boolean

'*******************************************************************************
' GetRelativePath (FUNCTION)
'
' PARAMETERS:
' (In) - sFromDirectory - String - Path for directory
' (In) - sToFile        - String - Path for file
'
' RETURN VALUE:
' String - The compacted path (empty if failed)
'
' DESCRIPTION:
' GetRelativePath demonstrates when given a directory path and a file path how
' to return the relative path from the directory to the file.
'*******************************************************************************
Public Function GetRelativePath(ByVal sFromDirectory As String, _
                                ByVal sToFile As String) As String
                                
    Dim sRelativePath   As String
    Dim bResult         As Boolean
    
    Const MAX_PATH As Long = 260
    
    sRelativePath = Space(MAX_PATH)
    
    ' Set "Attr" to vbDirectory for directories and 0 for files
    bResult = PathRelativePathToW(StrPtr(sRelativePath), StrPtr(sFromDirectory), _
        vbDirectory, StrPtr(sToFile), 0)
    
    If bResult Then
        GetRelativePath = Left(sRelativePath, InStr(sRelativePath, Chr(0)) - 1)
    End If
End Function

'*******************************************************************************
' GetCompactPath (FUNCTION)
'
' PARAMETERS:
' (In) - sPathToCompact - String - Path to compact
' (In) - lDeviceContext - Long   - Device context handle for widget text will
'                                  be displayed in
' (In) - lWidthInPixels - Long   - Pixel width to cram path into
'
' RETURN VALUE:
' String - Compacted path (empty if failed)
'
' DESCRIPTION:
' GetCompactPath demonstartes how to compact a long path name down into a small
' space in order to display it effectively in, say, a small text box.
'*******************************************************************************
Public Function GetCompactPath(ByVal sPathToCompact As String, _
                               ByVal lDeviceContext As Long, _
                               ByVal lWidthInPixels As Long) As String
                               
    Dim sCompactedPath  As String
    Dim bResult         As Boolean
    
    sCompactedPath = sPathToCompact
    bResult = PathCompactPathW(lDeviceContext, StrPtr(sCompactedPath), lWidthInPixels)
    
    If bResult Then
        GetCompactPath = Left(sCompactedPath, InStr(sCompactedPath, Chr(0)) - 1)
    End If
End Function
