VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "SaveInRegistry"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'-----------------------------------------------------------------------------
'Description: This DLL can be used to save the last opened files names in the registry for the application using the DLL
'Author     : Johann Granados Urea
'Date       : 15/03/2001
'             Use the "Initialize" method to establish the application name and the maximun number of file names to be saved
'             Use the "LastOpenedFiles" collection to access the last opened files names saved in the registry for the application using the DLL
'             Use the "SaveFileNameInRegistry" to save a file name in the registry for the application using the DLL
'CopyRight  : This DLL is provide "as is".  There is no warranty the DLL is going to work well in every situation.
'             Please feel free to improve this component.
'             If you do so, please let me know: johang@sol.racsa.co.cr
'             Thanks for using this component, sharing your knowledged and help me to improve my english
'-----------------------------------------------------------------------------
Option Explicit

'Application where the user is opening the files
Private cprv_strApplication
'This collection is used to remember the file names stored in the registry
Private cprv_coLastOpenedFiles As Collection
'Maximun number of files to be saved
Private cprv_bytMaxFiles As Byte

Public Sub Initialize(ByVal a_strApplication As String, _
                      ByVal a_bytMaxFiles As Byte)
    
    cprv_strApplication = a_strApplication
    cprv_bytMaxFiles = a_bytMaxFiles
    
    sc_GetLastOpenedFiles
    
End Sub

Public Sub SaveFileNameInRegistry(ByVal a_strFileName As String)
'Description: This procedure saves the file name received as parameter in the registry
'Parameters : a_strApplication > Name of the application where the file is being opened
'             a_strFile > Name of the file which is going to be saved in the registry
'Author     : Johann Granados Urea
'Date       : 15/03/2001 - 12:25 p.m.
Dim l_vntFile As Variant
Dim l_bytFileNumber As Byte
    
    If Not (fc_FileNameExists(a_strFileName)) Then
        SaveSetting cprv_strApplication, "LastOpenedFiles", "None", "None"
        'Delete the current information in the registry
        DeleteSetting cprv_strApplication, "LastOpenedFiles"
        
        'Is the collection empty?
        If cprv_coLastOpenedFiles.Count = 0 Then
           'Yes
           'Add the file name "normally"
           cprv_coLastOpenedFiles.Add a_strFileName
        Else
           'No
           'Add the file name as the first element in the collection
           cprv_coLastOpenedFiles.Add a_strFileName, , 1
        End If
           
        'Is the number of elements in the collection greater than the maximun allowed?
        If cprv_coLastOpenedFiles.Count > cprv_bytMaxFiles Then
           'Yes
           'Delete the last one (The collection will never grow more than one element beyond the maximun allowed)
           cprv_coLastOpenedFiles.Remove cprv_coLastOpenedFiles.Count
        End If
           
        'Save the file names in the registry
        l_bytFileNumber = 1
        For Each l_vntFile In cprv_coLastOpenedFiles
            SaveSetting cprv_strApplication, "LastOpenedFiles", "File" & l_bytFileNumber, l_vntFile
            l_bytFileNumber = l_bytFileNumber + 1
        Next
    Else
    
    End If
    
End Sub

Private Sub sc_GetLastOpenedFiles()
'Description    : This procedure add the last opened files names saved in the registry for the current application to the cprv_coLastOpenedFiles collection
'Parameters     :
'Author         : Johann Granados Urea
'Date           : 15/03/2001
Dim l_vntFiles As Variant
Dim l_intFileNumber As Integer
    
  Set cprv_coLastOpenedFiles = New Collection
    
  l_vntFiles = GetAllSettings(cprv_strApplication, "LastOpenedFiles")

  If Not (IsEmpty(l_vntFiles)) Then
    For l_intFileNumber = LBound(l_vntFiles, 1) To UBound(l_vntFiles, 1)
        cprv_coLastOpenedFiles.Add l_vntFiles(l_intFileNumber, 1)
    Next
End If
    
End Sub

Private Function fc_FileNameExists(ByVal a_strFileName As String) As Boolean
'Description    : This function check out if the file name received as parameter already exists in the file name collection
'Returned value : True > The file name does exist
'                 False > The file name does not exist
'Author         : Johann Granados Urea
'Date           : 15/03/2001 - 12:35 p.m.
Dim l_vntFile As Variant
Dim l_blnFound As Boolean
    
    'Set "the file name does not exist" as the default value
    l_blnFound = False
    
    For Each l_vntFile In cprv_coLastOpenedFiles
        'The current file name stored in "l_vntFile" is equal to the file name received as parameter
        If Trim$(l_vntFile) = Trim$(a_strFileName) Then
           'Yes
           'File name found, Exit the loop
           l_blnFound = True
           Exit For
        End If
    Next
    
    fc_FileNameExists = l_blnFound
    
End Function

Public Property Get LastOpenedFiles() As Collection
'Description: This property returns the last opened files collection
    
    Set LastOpenedFiles = cprv_coLastOpenedFiles
    
End Property
