Attribute VB_Name = "modHiScores"
'Project: HiScoreDemo.vbp
'Module: modHiScores.bas
'Author: Foyal S. Carter, Jr.
'E-mail: foyalcarter@mciworld.com
'Purpose:
'This module contains the
'HSType declaration ,Public variables,
'and the procedures for loading, saving,
'adding to, sorting and clearing the
'high scores list.

'Note: Read this module first.

'In order to preserve the accuracy of the scores
'when formatting them, all scores are of
'Type Variant which allows sub typing through use
'of the CDec function. This prevents the problems
'associated with binary writes of Double and
'Single precision floating point variables, and
'allows for larger scores than a Long would
'provide for.

Option Explicit

Private Type HSType             'user defined type to hold score info
    HSScore As Variant          'Allows use of Decimal subtype for formatting accuracy
    HSName As String * 3        'Player's initials
End Type

Public udtHiScores(0 To 9) As HSType   '10 scores of HSType

Public udtCurrScore As HSType      'current score of HSType

Public boolScoreAdded As Boolean    'flag for new score added

Sub LoadHiScores()

    Dim i As Integer
    Dim S As String
               
    'error checking
    On Error GoTo LoadHiScoresError
           
    'load 10 scores from registry
    'if non-existent, the default of "???0" is loaded
    
    For i = 0 To 9
                    
        S = GetSetting(App.Title, "HiScores", CStr(i), "???0")
        'App.Title -    the title of your application is the main key
        '"HiScores" -   the section of the main key to work with
        'CStr(i) -      the sub key names "0" to "9"
        '"???0" -       is the default value to load if
        '               one is not present in the registry
        
        
        With udtHiScores(i)
        
            'the first 3 characters are the users initials
            .HSName = Left(S, 3)
            'the remainder of the string is the score
            'sub type the variant using CDec...
            .HSScore = CDec(Mid(S, 4))
                 
        End With
        
    Next i
    
ExitLoadHiScores:
    
    Exit Sub

LoadHiScoresError:
    
    MsgBox Err.Description, , "LoadHiScores"
    Resume ExitLoadHiScores

End Sub

Sub SaveHiScores()

    Dim i As Integer
    Dim S As String
        
    'error checking
    On Error GoTo SaveHiScoresError
        
    'save the 10 high scores from the hiscores array
    For i = 0 To 9
                     
        With udtHiScores(i)
        
            'make the registry string from the
            'users initials and the high score
            S = .HSName & CStr(.HSScore)
                                 
        End With
        
        SaveSetting App.Title, "HiScores", CStr(i), S
        'App.Title -    the title of your application is the main key
        '"HiScores" -   the section of the main key to create
        'CStr(i) -      creates the sub key names "0" to "9"
        'S -            is the string value to save
                
    Next i

ExitSaveHiScores:
    
    Exit Sub

SaveHiScoresError:
    
    MsgBox Err.Description, , "SaveHiScores"
    Resume ExitSaveHiScores

End Sub

Sub SortHiScores()

    Dim i As Integer    'outer loop counter
    Dim j As Integer    'inner loop counter
    Dim k As Integer    'moving index
    Dim varHSLow As Variant 'comparison value
    Dim udtHSHold As HSType 'temp high score
    
    'error checking
    On Error GoTo SortHiScoreError
    
    'sort from low to high in order to place
    'tied scores lower in displayed list.
    
    'since only 10 items are to be sorted a
    'simple selection sort gets the job done
    
    'this sort works by searching the list for
    'the lowest item, swapping it with the one
    'at the top of the list, finding the next
    'lowest item, and swapping it with the
    'second, etc, until every item has been
    'swapped to a sorted position from low to high
    
    'from first position to last position - 1
    For i = 0 To 8
        
        'assign the score in the outer loop counter position
        'in the array to the comparison value
        'sub type the variant using CDec...
        varHSLow = CDec(udtHiScores(i).HSScore)
        'set the moving index to the outer loop counter position
        k = i
        
        'loop through the rest of the list and
        'find the lowest value remaining in the list
        'if one exists which is lower
        For j = i + 1 To 9
            
            'if the score at position j is lower than
            'the comparison value then
            If udtHiScores(j).HSScore < varHSLow Then
                
                'reassign the comparison value and
                'sub type the variant using CDec...
                varHSLow = CDec(udtHiScores(j).HSScore)
                
                'change the moving index to the inner loop counter position
                k = j
            
            End If
            
        Next j
        
        'switch the lowest score found (position k)
        'into the current location (position i)
        
        With udtHSHold
            'assign the data in position k to the temp score
            'sub type the variant using CDec...
            .HSScore = CDec(udtHiScores(k).HSScore)
            .HSName = udtHiScores(k).HSName
        
        End With
        
        With udtHiScores(k)
            'assign the data in position i to position k
            'sub type the variant using CDec...
            .HSScore = CDec(udtHiScores(i).HSScore)
            .HSName = udtHiScores(i).HSName
        
        End With
        
        With udtHiScores(i)
            'assign the data in the temp score to position i
            'sub type the variant using CDec...
            .HSScore = CDec(udtHSHold.HSScore)
            .HSName = udtHSHold.HSName
        
        End With
        
    Next i
     
       
ExitSortHiScores:
    
    Exit Sub
    
SortHiScoreError:

    MsgBox Err.Description, , "SortHiScores"
    Resume ExitSortHiScores
    
End Sub

Sub AddHiScore()
    
    'error checking
    On Error GoTo AddHiScoreError
    
    'replace the score with the lowest
    'value with the new score
    
    With udtHiScores(0)
        'sub type the variant using CDec...
        .HSScore = CDec(udtCurrScore.HSScore)
        .HSName = ""
    End With
    
    'then resort the array
    SortHiScores

    boolScoreAdded = True

ExitAddHiScore:

    Exit Sub

AddHiScoreError:

    MsgBox Err.Description, , "AddHiScore"
    Resume ExitAddHiScore

End Sub

Sub CheckHiScores()
    
    Dim i As Integer
    
    'error checking
    On Error GoTo CheckHiScoresError
    
    'loop through the high scores and if the new
    'score is higher than any one in the list then
    'add it to the list
    
    'loop from high to low to insure
    'a tied score won't be replaced
    For i = 9 To 0 Step -1
        
        'if the new score is higher than one in the list then
        If udtCurrScore.HSScore > udtHiScores(i).HSScore Then
            
            'add the score
            AddHiScore
            
            'and exit the loop
            Exit For
            
        End If
    
    Next i
    
ExitCheckHiScores:

    Exit Sub
    
CheckHiScoresError:
    
    MsgBox Err.Description, , "CheckHiScores"
    Resume ExitCheckHiScores
    
End Sub


Sub ClearHiScores()
    
    'error checking
    On Error GoTo ClearHiScoresError
    
    'this call deletes all registry settings
    'under this key and section
    
    DeleteSetting App.Title, "HiScores"
    'App.Title -    the title of your application is the main key
    '"HiScores" -   the section of the main key containing the scores
        
        
ExitClearHiScores:
    
    Exit Sub
    
ClearHiScoresError:
    
    MsgBox Err.Description, , "ClearHiScores"
    Resume ExitClearHiScores
    
End Sub
