VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CRainbowText"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' file    : CRainbowText.cls
' revised : 2001 April 30
' project : mRainbowText.vbp
' author  : Amanda Gunther
' email   : redbird77@earthlink.net
' www     : http://home.earthlink.net/~redbird77

Option Explicit

Private Const HSB_MAX = 100

Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" ( _
    Destination As Any, _
    Source As Any, _
    ByVal Length As Long)

Public Enum eLineBreak
    lbHtmlBreak
    lbStartNewLine
End Enum

Public Enum eOutputType
    otHtml
    'otMessageBoard
End Enum

Private m_eOutputType As eOutputType
Private m_eLineBreak  As eLineBreak
Private m_zHue        As Single
Private m_zSat        As Single
Private m_zBri        As Single
Private m_zCycles     As Single
Private m_sText       As String
Private m_rtfPreview  As RichTextBox

Private Sub Class_Initialize()
    
    m_zHue = 100
    m_zSat = 100
    m_zBri = 100
    m_zCycles = 100
    
End Sub

Public Property Get Text() As String
    Text = m_sText
End Property

Public Property Let Text(NewValue As String)
    m_sText = NewValue
End Property

Public Property Get Hue() As Single
    Hue = m_zHue
End Property

Public Property Let Hue(NewValue As Single)
    m_zHue = NewValue
End Property

Public Property Get Saturation() As Single
    Saturation = m_zSat
End Property

Public Property Let Saturation(NewValue As Single)
    m_zSat = NewValue
End Property

Public Property Get Brightness() As Single
    Brightness = m_zBri
End Property

Public Property Let Brightness(NewValue As Single)
    m_zBri = NewValue
End Property

Public Property Get Cycles() As Single
    Cycles = m_zCycles
End Property

Public Property Let Cycles(NewValue As Single)
    m_zCycles = NewValue
End Property

Public Property Get LineBreak() As eLineBreak
    LineBreak = m_eLineBreak
End Property

Public Property Let LineBreak(NewValue As eLineBreak)
    m_eLineBreak = NewValue
End Property
' ---
Public Property Get OutputType() As eOutputType
    OutputType = m_eOutputType
End Property

Public Property Let OutputType(NewValue As eOutputType)
    m_eOutputType = NewValue
End Property

Public Property Set PreviewRichTextBox(NewValue As RichTextBox)
    ' TODO: Replace with a Text and a TextRainbow (similar to TextRTF) property.
    Set m_rtfPreview = NewValue
End Property

Public Function Create() As String

    Dim i         As Long
    Dim j         As Long
    
    Dim lCol      As Long
    Dim zInc      As Single
    
    Dim bytChr()  As Byte
    Dim sChr      As String
    Dim sChrHtm   As String
    Dim lCount    As Long
    
    Dim sBreak    As String
    Dim sLines()  As String
    
    Dim sTagStart As String
    Dim sTagMid   As String
    Dim sTagEnd   As String
    
    Dim sHTM      As String
    Dim fsHTM     As CFastString
    
    Set fsHTM = New CFastString
    
    m_rtfPreview.Text = ""
    
    sBreak = IIf(m_eLineBreak = lbHtmlBreak, "<br>", vbCrLf)
    
    ' Will change if other output formats.
'    sTagStart = "<font color="""
'    sTagMid = """>"
'    sTagEnd = "</font>"

    ' Get number of printable non-whitepace characters.
    bytChr = StrConv(m_sText, vbFromUnicode)

    For i = 0 To UBound(bytChr)
        If bytChr(i) > 32 And (bytChr(i) < 127 Or bytChr(i) > 160) Then _
            lCount = lCount + 1
    Next

    ' Calculate hue's increment.
    If (lCount - 1) > 0 Then zInc = m_zCycles / (lCount - 1)
    
    ' All of the 3 following variables were passed in in the 0 - 100 range.
    ' To use a 0 - 255 range divide by 255 instead of 100.
    m_zHue = m_zHue / HSB_MAX
    m_zSat = m_zSat / HSB_MAX
    m_zBri = m_zBri / HSB_MAX
    
    sLines = Split(m_sText, vbCrLf)
    
    For i = 0 To UBound(sLines)
    
        ' Access chars via array, no need for the Mid$ function.
        bytChr = StrConv(sLines(i), vbFromUnicode)
        
        For j = 0 To UBound(bytChr)
            ' Get the character.
            sChr = Chr$(bytChr(j))
            
            ' If the char is whitespace, do not calculate a color or
            ' increment the hue.
            If sChr = " " Or sChr = vbTab Then
                fsHTM.Append sChr
            Else
                ' Determine the character's color.
                lCol = HSBtoLNG(m_zHue, m_zSat, m_zBri)
                
                sChrHtm = sChr
                
                If sChrHtm = ">" Then
                    sChrHtm = "&gt;"
                ElseIf sChrHtm = "<" Then
                    sChrHtm = "&lt;"
                ElseIf sChrHtm = "&" Then
                    sChrHtm = "&amp;"
                End If
                
                ' Create the HTML font tag.
                fsHTM.Append "<font color=""" & LNGtoHEX(lCol) & """>" & _
                             sChrHtm & "</font>"
              
                ' Increment the hue.
                m_zHue = m_zHue + zInc
            End If
            
            ' Create the color preview.
            With m_rtfPreview
                .SelStart = Len(.Text)
                .SelColor = lCol
                .SelText = sChr
            End With
        Next
        
        ' Insert line breaks.
        If i < UBound(sLines) Then
            fsHTM.Append sBreak
            
            With m_rtfPreview
                .SelStart = Len(.Text)
                .SelText = vbCrLf
            End With
        End If
    Next
    
    Create = fsHTM.Buffer
    
    Set fsHTM = Nothing

End Function

' ---------------------------------------------------------------------------
' Color Functions
' ---------------------------------------------------------------------------

Private Function LNGtoHEX(ByVal lColor As Long) As String

    Dim B(2) As Byte
    
    CopyMemory B(0), lColor, 3

    ' You can't just Hex$ a long to get a web-ready hex triplet color string,
    ' it'll be rerversed (i.e. ff6034 instead of 3460ff).
    LNGtoHEX = "#" & Right$("00000" & LCase$(Hex$(RGB(B(2), B(1), B(0)))), 6)
    
End Function

Private Function HSBtoLNG(ByVal H As Single, _
                          ByVal S As Single, _
                          ByVal B As Single) As Long
                         
' PURPOSE:
' HSBtoLNG allows you to specify a color value in HSB terms and returns
' a the same value in long terms.  The function is useful for creating
' a rainbow effect.  Just start the hue at zero, and draw a line across
' the form, with the Color argument of the Line function being
' HSB(hue, 1, 1).  Now increment the hue (maybe by .01), move down a pixel
' and draw another line with the new color.
'
' INPUT:
' H, S, B - single values representing the hue, saturation and brightness
'           of a given color.
' OUTPUT:
' The function returns given color represented as a long, which can then
' be used most anywhere a color value is needed.
'
' SAMPLE CALL:
' Text1.BackColor = HSBtoLNG(.5, 1, 1) ' Text1's BackColor is a vbCyan
              
    Dim p As Single, q As Single, t As Single, f As Single, i As Long
    Dim lRet As Long
    
    ' The modifications below allow one to feed in progressively higher
    ' values for H, S, and B without needing to worry about staying in
    ' the 0 - 1 range.  The H values form a "circle", the S and B values
    ' go from 0 - 1 - 0, a kind of "ladder".
    '
    ' An error would occur though is H, S or B were to become larger than a
    ' single type can hold.
    H = H - Int(H)
    S = Abs((Int(S) And 1) - S + Int(S))
    B = 255 * Abs((Int(B) And 1) - B + Int(B))
    
    If S = 0 Then
        lRet = RGB(B, B, B)
    Else
        H = H * 6
        i = Int(H)
        f = H - i
        H = Int(H)
        p = Int(B * (1 - S))
        q = Int(B * (1 - (S * f)))
        t = Int(B * (1 - (S * (1 - f))))

        Select Case i
            Case 0: lRet = RGB(B, t, p)
            Case 1: lRet = RGB(q, B, p)
            Case 2: lRet = RGB(p, B, t)
            Case 3: lRet = RGB(p, q, B)
            Case 4: lRet = RGB(t, p, B)
            Case Else: lRet = RGB(B, p, q)
        End Select
    End If
    
    HSBtoLNG = lRet
    
End Function
