Attribute VB_Name = "mRainbowText"
' file    : mRainbowText.bas
' revised : 2000-09-11
' project : mRainbowText.vbp
' author  : Amanda Gunther
' email   : redbird77@earthlink.net
' www     : http://home.earthlink.net/~redbird77

Option Explicit

Private Const HSB_MAX = 100

Private Declare Sub CopyMemory Lib "kernel32" _
    Alias "RtlMoveMemory" ( _
    Destination As Any, _
    Source As Any, _
    ByVal Length As Long)
    
' ----------------------------------------------------------------------------

Public Function Rainbow(sTxt As String, _
                        zCycles As Single, _
                        zHue As Single, _
                        zSat As Single, _
                        zBri As Single, _
                        rtfHold As RichTextBox) As String

    Dim i        As Long
    Dim j        As Long
    Dim lCol     As Long
    Dim zInc     As Single
    Dim bytChr() As Byte
    Dim sChr     As String
    Dim sHTM     As String
    Dim lCount   As Long
    Dim sLines() As String
    Dim sBreak   As String
    
    'rtfHold.TextRTF = ""
    
    sBreak = IIf(fMain.optLineBreak(0).Value, "<br>", vbCrLf)
    
    bytChr = StrConv(sTxt, vbFromUnicode)

    For i = 0 To UBound(bytChr)
        ' 32 = 20h = space
        If bytChr(i) > 32 And (bytChr(i) < 127 Or bytChr(i) > 160) Then _
            lCount = lCount + 1
    Next
    
    ' Calculate the hue's increment based on printable characters & no whitespace.
    If (lCount - 1) <> 0 Then
        zInc = zCycles / (lCount - 1)
    Else
        zInc = 0
    End If
    
    ' All of the 3 following variables were passed in in the 0 - 100 range.
    ' To use a 0-255 range divide by 255 instead of 100.
    zHue = zHue / HSB_MAX
    zSat = zSat / HSB_MAX
    zBri = zBri / HSB_MAX
    
    sLines = Split(sTxt, vbCrLf)
    
    For i = 0 To UBound(sLines)
    
        ' Access chars via array, no need for the Mid$ function.
        bytChr = StrConv(sLines(i), vbFromUnicode)
        
        For j = 0 To UBound(bytChr)
        
            sChr = Chr$(bytChr(j))
            
            If sChr = " " Or sChr = vbTab Then
                sHTM = sHTM & sChr
            Else
            
                ' Determine the character's color.
                lCol = HSBtoLNG(zHue, zSat, zBri)
            
                ' Create the HTML font tag.
                sHTM = sHTM & "<font color=""" & _
                       LNGtoHEX(lCol) & _
                       """>" & sChr & "</font>"
              
                ' Increment the hue.
                zHue = zHue + zInc
            End If
            
            ' Create the color preview.
            With rtfHold
                .SelStart = Len(.Text)
                .SelColor = lCol
                .SelText = sChr
            End With
        Next
        
        ' Insert line breaks.
        If i < UBound(sLines) Then
            sHTM = sHTM & sBreak
            
            With rtfHold
                .SelStart = Len(.Text)
                .SelText = vbCrLf
            End With
            
        End If
        
    Next

    Rainbow = sHTM

End Function

Private Function LNGtoHEX(ByVal lColor As Long) As String

    Dim B(2) As Byte
    
    CopyMemory B(0), lColor, 3

    ' You can't just Hex$ a long to get a web-ready hex triplet color string,
    ' it'll be rerversed (i.e. ff6034 instead of 3460ff).
    LNGtoHEX = "#" & Right$("00000" & LCase$(Hex$(RGB(B(2), B(1), B(0)))), 6)
    
End Function

Private Function HSBtoLNG(ByVal H As Single, _
                          ByVal S As Single, _
                          ByVal B As Single) As Long
                         
' PURPOSE:
' HSBtoLNG allows you to specify a color value in HSB terms and returns
' a the same value in long terms.  The function is useful for creating
' a rainbow effect.  Just start the hue at zero, and draw a line across
' the form, with the Color argument of the Line function being
' HSB(hue, 1, 1).  Now increment the hue (maybe by .01), move down a pixel
' and draw another line with the new color.
'
' INPUT:
' H, S, B - single values representing the hue, saturation and brightness
'           of a given color.
' OUTPUT:
' The function returns given color represented as a long, which can then
' be used most anywhere a color value is needed.
'
' SAMPLE CALL:
' Text1.BackColor = HSBtoLNG(.5, 1, 1) ' Text1's BackColor is a vbCyan
              
    Dim p As Single, q As Single, t As Single, f As Single, i As Long
    Dim lRet As Long
    
    ' The modifications below allow one to feed in progressively higher
    ' values for H, S, and B without needing to worry about staying in
    ' the 0 - 1 range.  The H values form a "circle", the S and B values
    ' go from 0 - 1 - 0, a kind of "ladder".
    '
    ' An error would occur though is H, S or B were to become larger than a
    ' single type can hold.
    H = H - Int(H)
    S = Abs((Int(S) And 1) - S + Int(S))
    B = 255 * Abs((Int(B) And 1) - B + Int(B))
    
    If S = 0 Then
        lRet = RGB(B, B, B)
    Else
        H = H * 6
        i = Int(H)
        f = H - i
        H = Int(H)
        p = Int(B * (1 - S))
        q = Int(B * (1 - (S * f)))
        t = Int(B * (1 - (S * (1 - f))))

        Select Case i
            Case 0: lRet = RGB(B, t, p)
            Case 1: lRet = RGB(q, B, p)
            Case 2: lRet = RGB(p, B, t)
            Case 3: lRet = RGB(p, q, B)
            Case 4: lRet = RGB(t, p, B)
            Case Else: lRet = RGB(B, p, q)
        End Select
    End If
    
    HSBtoLNG = lRet
    
End Function
