Attribute VB_Name = "modLuhn"
Option Explicit

' Copyright  1999-2001 Sualeh Fatehi. All Rights Reserved.
'
' Permission to use, copy, modify, and redistribute this software
' and its documentation for personal, non-commercial use is hereby
' granted provided that this copyright notice and appropriate
' documentation appears in all copies. This software may not be
' distributed for fee or as part of commercial, "shareware," and/or
' not-for-profit endevors including, but not limited to, CD-ROM
' collections, online databases, and subscription services without
' specific license.

Public Function ACType(ByVal ACNumber As String) As String
' Author:         sfatehi
' Date:           1999-Sep-23
' Version:        1.0
' Last Modified:
' Description:    Returns the account type
' Arguments:      ACNumber: Credit card account number
' Return Value:   Account type, if valid, or an empty string
' Error Handling:
' Limitations:
' Known Bugs:

Const strRoutine = "ACType"
On Error GoTo ACTypeErr

Dim strAccountType              As String
Dim strAccountNumber            As String
Dim intPrefix                   As Integer

    ' Check if the number is valid
    If Not IsValid(ACNumber) Then
        ACType = vbNullString
        Exit Function
    End If

    ' Attempt to find the account type
    strAccountNumber = Numeric(ACNumber)
    strAccountType = vbNullString
    ' 1. MasterCard
    If Len(strAccountNumber) = 16 Then
        intPrefix = Val(Mid$(strAccountNumber, 1, 2))
        If intPrefix >= 51 And intPrefix <= 55 Then
            strAccountType = "MasterCard"
        End If
    End If
    ' 2. Visa
    If Len(strAccountNumber) >= 13 And Len(strAccountNumber) <= 16 Then
        intPrefix = Val(Mid$(strAccountNumber, 1, 1))
        If intPrefix = 4 Then
            strAccountType = "Visa"
        End If
    End If
    ' 3. American Express
    If Len(strAccountNumber) = 15 Then
        intPrefix = Val(Mid$(strAccountNumber, 1, 2))
        If intPrefix = 34 Or intPrefix = 37 Then
            strAccountType = "American Express"
        End If
    End If
    ' 4. Diners Club
    If Len(strAccountNumber) = 14 Then
        intPrefix = Val(Mid$(strAccountNumber, 1, 2))
        If intPrefix = 36 Or intPrefix = 38 Then
            strAccountType = "Diners Club"
        End If
        intPrefix = Val(Mid$(strAccountNumber, 1, 3))
        If intPrefix >= 300 And intPrefix <= 305 Then
            strAccountType = "Diners Club"
        End If
    End If
    ' 5. Discover
    If Len(strAccountNumber) = 16 Then
        intPrefix = Val(Mid$(strAccountNumber, 1, 4))
        If intPrefix = 6011 Then
            strAccountType = "Discover"
        End If
    End If

    ACType = strAccountType

ACTypeErr:

End Function


Public Function IsValid(ByVal ACNumber As String) As Boolean
' Author:         sfatehi
' Date:           1999-Sep-23
' Version:        1.0
' Last Modified:
' Description:    Checks if a credit card is valid, using the Luhn
'                 method
' Arguments:      ACNumber: Credit card account number
' Return Value:   Whether the card number is valid or not
' Error Handling:
' Limitations:
' Known Bugs:


Const strRoutine = "IsValid"
On Error GoTo IsValidErr

Dim i               As Integer
Dim iLuhn           As Long
Dim strNumber       As String

    ' Do quick checks on the account number
    strNumber = Numeric(ACNumber)
    If Val(strNumber) = 0 Then
        IsValid = False
        Exit Function
    End If
    If Len(strNumber) < 13 Or Len(strNumber) > 16 Then
        IsValid = False
        Exit Function
    End If
    ' Format the number as a 16 digit number
    strNumber = Format$(strNumber, String$(16, "0"))

    iLuhn = 0
    For i = 16 To 1 Step -1
        iLuhn = iLuhn + SumDigits(GetDigit(strNumber, i) * (2 - ((17 - i) Mod 2)))
    Next i

    If (iLuhn Mod 10) = 0 Then
        IsValid = True
    Else
        IsValid = False
    End If

IsValidErr:

End Function

Public Function Numeric(ByVal Number As String) As String
' Author:         sfatehi
' Date:           1999-Sep-23
' Version:        1.0
' Last Modified:
' Description:    Removes non-numeric characters from a string
' Arguments:      Number: a number
' Return Value:   The numeric number
' Error Handling:
' Limitations:
' Known Bugs:

Const strRoutine = "Numeric"
On Error GoTo Numeric

Dim i                   As Integer
Dim strNumeric          As String
Dim strCharacter        As String

    strNumeric = vbNullString
    For i = 1 To Len(Number)
        strCharacter = Mid$(Number, i, 1)
        If IsNumeric(strCharacter) Then
            strNumeric = strNumeric + strCharacter
        End If
    Next i

    Numeric = strNumeric

Numeric:

End Function


Private Function GetDigit(ByVal Number As String, ByVal Index As Integer) As Integer
' Author:         sfatehi
' Date:           1999-Sep-23
' Version:        1.0
' Last Modified:
' Description:    Gets a certain digit in the number
' Arguments:      Number: a number
' Return Value:   The digit
' Error Handling:
' Limitations:
' Known Bugs:

Const strRoutine = "GetDigit"
On Error GoTo GetDigitErr

Dim strNumber           As String
Dim intDigit            As Integer

    strNumber = Number
    If Index <= Len(strNumber) Then
        intDigit = Val(Mid$(strNumber, Index, 1))
    Else
        intDigit = 0
    End If

    GetDigit = intDigit

GetDigitErr:

End Function

Private Function SumDigits(ByVal Number As Integer) As Integer
' Author:         sfatehi
' Date:           1999-Sep-23
' Version:        1.0
' Last Modified:
' Description:    Adds the digits in a number
' Arguments:      Number: a number
' Return Value:   The sum of the digits
' Error Handling:
' Limitations:
' Known Bugs:

Const strRoutine = "SumDigits"
On Error GoTo SumDigitsErr

Dim i                   As Integer
Dim strNumber           As String
Dim intSum              As Integer

    intSum = 0
    For i = 1 To 5
        intSum = intSum + GetDigit(Number, i)
    Next i

    SumDigits = intSum

SumDigitsErr:

End Function

