Attribute VB_Name = "modToken"
Option Explicit
Option Base 0

Private aintDelimiterPosition() As Integer
Private astrTokenString() As String

Public Function TokenizeString(strDelimiter As String, _
                               strInputString As Variant) As Integer
            
'USAGE: TokenizeString(Delimiter Char(s), Search String )
'RETURNS: integer represent number of tokens found in input string delimited by input delimiter char
'NOTE: Treats consecutive parsing characters as one.  Saves tokens in token array astrTokenString.
'       Saves position of all delimiters in delimeter array aintDelimiterPosition
            
            
'EXAMPLE: TokenizeString(",", "A,B,C,D,E,,F")
'RETURNS: "6" to the Debug Window and stores A,B,C,D,E,F to astrTokenString(1 to 6)

    Dim intCurrentPosition As Integer
    Dim intParseToPosition As Integer
    Dim intNumDelimiters As Integer
    Dim intNumTokens As Integer
    Dim intDelimeterStart As Integer
    Dim intDelimeterLen As Integer
    Dim i As Integer
    
    intCurrentPosition = 1
    intNumDelimiters = 0
    intNumTokens = 0
  
    Erase aintDelimiterPosition()
    Erase astrTokenString()
    
    ReDim Preserve aintDelimiterPosition(intNumDelimiters)
    aintDelimiterPosition(0) = 0          'Add imaginary delimter to start of string at position 0
    
    Do      'Loop through string search for delimiter, and store location of delimiter in array
    
        intParseToPosition = InStr(intCurrentPosition, strInputString, strDelimiter)
    
        If (intParseToPosition > 0) Then
        
            intNumDelimiters = intNumDelimiters + 1                         'Increment delimiter count
            ReDim Preserve aintDelimiterPosition(intNumDelimiters)          'Add one position the delimiter position array and keep old contents
            aintDelimiterPosition(intNumDelimiters) = intParseToPosition    'Store delimiter position in newly created position
            intCurrentPosition = intParseToPosition + 1                     'Increment to next position in string
            
        End If
    
    Loop Until ((intCurrentPosition > Len(strInputString)) Or (intParseToPosition = 0))
    
    
    intNumDelimiters = intNumDelimiters + 1
    ReDim Preserve aintDelimiterPosition(intNumDelimiters)
    aintDelimiterPosition(intNumDelimiters) = Len(strInputString) + 1    'Add imaginary delimter to end of string
          
    intDelimeterStart = aintDelimiterPosition(0)            'Start with position of first delimiter
    
    For i = 1 To intNumDelimiters                           'Loop through all delimiter positions
    
        intDelimeterLen = aintDelimiterPosition(i) - intDelimeterStart
        
        If (intDelimeterLen > 1) Then                       'If length between two delimeters > 1 then found token
        
            intNumTokens = intNumTokens + 1
            ReDim Preserve astrTokenString(intNumTokens)
            astrTokenString(intNumTokens) = Mid(strInputString, intDelimeterStart + 1, intDelimeterLen - 1)
        End If
            
        intDelimeterStart = aintDelimiterPosition(i)        'Save new start position
                        
    Next
      
    TokenizeString = intNumTokens
  
End Function

Public Function GetDelimiterCount() As Integer
'USAGE: GetDelimiterCount()
'REQUIRES: that TokenizeString() be called first
'RETURNS: an integer representing the number of delimiters found in strInputString
'          or 0 if TokenizeString was not called or if no delimiters were found.
'          *** Does not inlcude imaginary delimiters added to ends of input string

    GetDelimiterCount = UBound(aintDelimiterPosition) - 1

End Function


Public Function DelimiterPosition(intTokenNum As Integer) As Integer

'USAGE: DelimiterPosition(integer delimiter number)
'REQUIRES: that TokenizeString() be called first
'RETURNS: an integer representing the location where the 3rd delimiter was found in strInputString
'          or 0 if intTokenNum is out of range

    On Error GoTo errDelimiterPosition

    If (UBound(aintDelimiterPosition) > 1) Then
    
        DelimiterPosition = aintDelimiterPosition(intTokenNum)  'Return the position of the Token in the string
    
    Else
    
        GoTo errDelimiterPosition
    
    End If
    
    Exit Function
    
errDelimiterPosition:

    DelimiterPosition = 0

End Function


Public Function GetToken(intTokenNum As Integer) As String

'USAGE: GetToken(5)
'REQUIRES: that TokenizeString() be called first
'RETURNS: the 5th token found in strInputString or the empty string if intTokenNum is out of range

    On Error GoTo errGetToken

    GetToken = astrTokenString(intTokenNum)  'Return the position of the Token in the string
    Exit Function
    
errGetToken:

    GetToken = ""

End Function


